function [Is, Js,Its,Itt,Ittt,caps, dems, dists,lbs,ubs, dists1,dists2,dists3,dists4,dists5,caps1,caps3,caps2] =MRI(C,model,gMA,vMA,U,KeyGene);
% Shaghayegh Yazdanpanah 
%July 12, 2022
% MRI, a method for investigation of reprogramming by adaptation score calculation

% INPUT
% C                Constant parameter for conversion of expression level to upper bound of reactions               
% model        Genome-scale metabolic model
% gMA           Gene names
% vMA           Gene expressions
%U                 Constant parameter

% OPTIONAL INPUT
%KeyGene     Gene or a reaction that adaptation score is calculated under the condition of its optimal value

% OUTPUT
% Is               Number of TRFBA equations
% Js               Number of TRFBA variables
% Its              Number of Eq. (3) 
% Itt              Number of Eq. (5) and Eq. (6)
% Ittt             Number of Eq. (7)
% caps          b, Right hand side = dx/dt
% dems         c, Objective coefficients
%dists           A, TRFBA LHS matrix 
% lbs             Lower bounds
% ubs            Upper bounds
% dists1        A1, Alfa coefficient matrix in Eq. (3)
% dists2        A2, z coefficient matrix in Eq. (3)
% dists3        A3, reversible reaction coefficient matrix in Eq. (5) and Eq. (6)         
% dists4        A4, binary variable (w) coefficient matrix in Eq. (5) and Eq. (6)   
% dists5        A5, Eq. (7)
% caps1        Right hand side of Eq. (3)
% caps2        Right hand side of Eq. (5) and Eq. (6)
% caps3        Right hand side of Eq. (7)
% optimal     Optimal amount of KeyGene

[model,RegulatedGenes,matchRev,nRxnsR,nMetsR]=call_TRFBA2(C,model,gMA,vMA);
if exist('KeyGene','var')
    model.c(:)=0;
    model=changeObjective(model,KeyGene);
end
A=full(model.S);b=model.b;c=model.c;lb=model.lb;ub=model.ub;

%% GAMS

[junk,pN] = size(b');
[junk,dN] = size(c');
Is.name = 'I';
Is.val  = [ linspace(1,pN,pN) ]';
Is.type = 'set';

Js.name = 'J';
Js.val  = [ linspace(1,dN,dN) ]';
Js.type = 'set';


caps.name = 'b';
caps.type = 'parameter';
caps.val = b';
caps.form = 'full';
caps.dim = 1;

dems.name = 'c';
dems.type = 'parameter';
dems.val = c';
dems.form = 'full';
dems.dim = 1;

dists.name = 'A';
dists.type = 'parameter';
dists.val = A;
dists.form = 'full';
dists.dim = 2;

lbs.name = 'lb';
lbs.type = 'parameter';
lbs.val = lb;
lbs.form = 'full';
lbs.dim = 1;
ubs.name = 'ub';
ubs.type = 'parameter';
ubs.val = ub;
ubs.form = 'full';
ubs.dim = 1;

% Inequality defining Eq. (3)

U=10^20;tt=1;
for i=nRxnsR+1:nRxnsR+length(RegulatedGenes)
    A1(tt,i)=1;A2(tt,i)=-U;b1(tt,1)=0;tt=tt+1;
end

Its.name = 'It';
Its.val  = [ linspace(1,length(RegulatedGenes),length(RegulatedGenes)) ]';
Its.type = 'set';

dists1.name = 'A1';
dists1.type = 'parameter';
dists1.val = A1;
dists1.form = 'full';
dists1.dim = 2;

dists2.name = 'A2';
dists2.type = 'parameter';
dists2.val = A2;
dists2.form = 'full';
dists2.dim = 2;

caps1.name = 'b1';
caps1.type = 'parameter';
caps1.val = b1';
caps1.form = 'full';
caps1.dim = 1;

% Set of Inequalities definig Eq. (5) to Eq. (7)

k=1;kk=1;A5=zeros(1,dN);
for j=1:length(matchRev)
    if matchRev(j)~=0
        a=find(ismember(model.rxns,model.rxns(j)));
        for g=1:length(a)
            A3(k,j)=1;
            A3(k+1,matchRev(a(g)))=1;
            A4(k,j)=-U;
            A4(k+1,matchRev(a(g)))=-U;
            b2(k,1)=0;b2(k+1,1)=0;
            A5(kk,j)=1; A5(kk,matchRev(a(g)))=1;
            b3(kk,1)=1;
            k=k+2;kk=kk+1;
            matchRev(matchRev(a(g)))=0;
        end
    end
end

dists3.name = 'A3';
dists3.type = 'parameter';
dists3.val = A3;
dists3.form = 'full';
dists3.dim = 2;

dists4.name = 'A4';
dists4.type = 'parameter';
dists4.val = A4;
dists4.form = 'full';
dists4.dim = 2;

dists5.name = 'A5';
dists5.type = 'parameter';
dists5.val = A5;
dists5.form = 'full';
dists5.dim = 2;

caps2.name = 'b2';
caps2.type = 'parameter';
caps2.val = b2';
caps2.form = 'full';
caps2.dim = 1;

caps3.name = 'b3';
caps3.type = 'parameter';
caps3.val = b3';
caps3.form = 'full';
caps3.dim = 1;

Itt.name = 'Itt';
Itt.val  = [ linspace(1,k-2,k-2) ]';
Itt.type = 'set';

Ittt.name = 'Ittt';
Ittt.val  = [ linspace(1,kk-1,kk-1) ]';
Ittt.type = 'set';

%Adaptation score under maximum usage of a gene or reaction

if exist('KeyGene','var')
    
    wgdx ('trdat3', Is, Js,Its,Itt,Ittt,caps, dems, dists,lbs,ubs, dists1,dists2,dists3,dists4,dists5,caps1,caps3,caps2);
    
    gams_output = 'std';
    gams('mipTRFBA mip=cplex');
    
    % now get the outputs from the GDX file produced by the GAMS run
    
    rs = struct('name','modelStat','form','full');
    r = rgdx ('trsol2', rs);
    modelstat = r.val;
    %
    rs.name = 'solveStat';
    r = rgdx ('trsol2', rs);
    solvestat = r.val;
    %
    rs.name = 'z';
    rs.field = 'l';
    r = rgdx ('trsol2', rs);
    z = r.val;
    optimal=z;
    
    rs.name = 'x';
    r = rgdx ('trsol2', rs);
    x = r.val;
    
    rs.name = 'w';
    r = rgdx ('trsol2', rs);
    w = r.val;
    
    model = changeRxnBounds(model,KeyGene,z,'b');
    lb=model.lb;ub=model.ub;
    
    lbs.name = 'lb';
    lbs.type = 'parameter';
    lbs.val = lb;
    lbs.form = 'full';
    lbs.dim = 1;
    ubs.name = 'ub';
    ubs.type = 'parameter';
    ubs.val = ub;
    ubs.form = 'full';
    ubs.dim = 1;
end


