$Title MIP with sets and parameters from GDXMRW - no UELS (TRMAT2,SEQ=10XX)

$Ontext
We solve an LP model in GAMS with the data coming from Matlab: c.f. the
file trrun2.m, which runs this file as one of the steps below.

The style we use here breaks the process into smaller steps:
  1. Write the sets and parameters from Matlab to the first GDX file using
     wgdx(). The resulting GDX file can be viewed without running GAMS -
     this makes debugging data issues easier.
  2. Run the GAMS model, reading data from the first GDX file and writing
     the results to a second GDX file.  The modeling and debugging can
     take place without running Matlab or even having it installed, and
     subsequent problems can be reproduced trivially.
  3. Read the solution from the second GDX file into Matlab.

The disadvantage of this scheme compared to one where all data is passed
via the gams() call is that it doesn't have the feel of a
subroutine call.  In addition to the advantages explained above, it is
also easier to read and write data at both compile and execution time
using this scheme.

Contributor: Steven Dirkse, November 2010

$Offtext

sets
  i   'canning plants'
  j   'markets'
It
Itt
Ittt
  ;
parameters  A(i,j),b(i),c(j),lb(j),ub(j),A1(It,j),b1(It),A2(It,j),b2(Itt),b3(Ittt),A5(Ittt,j),A3(Itt,j),b2(Itt),A4(Itt,j);
$gdxin trdat3
$load i j It Itt Ittt A b lb c ub A1 A2 A5 b1 b3 A3 A4 b2
$gdxin
$onUNDF
variable z;
variable x(j);
binary variable w(j);
binary variable y(j);
equation cost, dual(i),dual2(It),dual3(Itt),dual5(Ittt);
cost.. z =e= sum(j,w(j));
dual(i)..   sum(j, A(i,j)*x(j)) =e= b(i);
dual2(It)..   sum(j, A1(It,j)*x(j)+A2(It,j)*w(j)) =l= b1(It);
dual3(Itt)..   sum(j,A3(Itt,j)*x(j)+A4(Itt,j)*y(j)) =l= b2(Itt);
dual5(Ittt)..   sum(j,A5(Ittt,j)*y(j))=l= b3(Ittt);
$if exist matdata.gms $include matdata.gms
x.lo(j) = lb(j);
x.up(j) = ub(j);

model transport /all/;
solve transport using mip minimizing z;

scalars modelStat, solveStat;
modelStat = transport.modelstat;
solveStat = transport.solvestat;

execute_unload 'trsol2', modelStat, solveStat, z, x,w;
